#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/27 23:10
@File  : test_zentao.py
'''
import pytest

''' 测试提交BUG单在conftest.py中，但是本用例触发 '''

def test_login():
    ''' 测试登录
        1、判断账户正确性
        2、判断密码正确性
        3、提交数据
    '''
    assert True

def test_search():
    ''' 测试搜索
        1、输入数据
        2、点击搜索按钮
    '''
    assert True

def test_order():
    ''' 测试下单
        1、选择商品
        2、点击下单按钮
    '''
    assert True


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/9/2 1:21
@File  : test_xiaobai_case_submitBug.py
'''

'''
# filename = conftest.py 新增内容
from saf.utils.submitBugUtils import addZenTaoBUG
import pytest

TESTTYPE = 'sendMsg'

@pytest.mark.hookwrapper
def pytest_runtest_makereport(item, call):
    """
    :param item     : 测试的单元对象
    :param call     : 测试的步骤：when（setup, call, teardown）三个步骤
    outcome         : 用例测试结果对象
    """
    outcome = yield                     # 获取每一条用例的执行结果
    report = outcome.get_result()
    if report.outcome == 'failed':
        if 'submitBug' == TESTTYPE:
            doc = item.function.__doc__
            doc = str(doc).replace('\n', '<br>')
            addZenTaoBUG(title=item.function.__name__,
                          steps=f'{doc}预期结果：passed<br>测试结果：{report.outcome}')
'''


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/28 23:37
@File  : elementUtils.py
'''
from saf import WebDriver, By
from time import sleep

def find_element(driver: WebDriver = None,
                 by: str = By.XPATH,
                 value: str = '',
                 total_time: int = 30,
                 step_time: float = 0.5):
    '''
    通用定位元素方法
    :param driver       : 浏览器对象
    :param by           : 定位表达式
    :param value        : 定位表达式值
    :param total_time   : 定位超时时间（单位：秒）
    :param step_time    : 每次间隔定位时间（单位：秒）
    :return:
    '''
    while 1:
        if total_time > 0:
            if 0 == len(driver.find_elements(by=by, value=value)):
                sleep(step_time)
                total_time -= step_time
            else:
                return driver.find_element(by=by, value=value)
        else:
            break
    return None

def find_elements(driver: WebDriver = None, by: str = By.XPATH, value: str = ''):
    '''
    通用定位元素（复数）
    :param driver:
    :param by:
    :param value:
    :return:
    '''
    return driver.find_elements(by=by, value=value)


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/28 23:40
@File  : test_element.py
'''
from saf import *
from saf.utils.elementUtils import find_element
from saf.utils.imageUtils import image2str

c = selenium_webdriver.Chrome(ChromeDriverManager().install())
c.get('http://mail.xiaobai.com:8080/Center/Index/login')
find_element(driver=c, value='//*[@name="captcha"]').send_keys(image2str(driver=c, value='//*[@id="captcha"]'))


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/24 1:37
@File  : image_utils.py
'''
''' 将图片验证码转为字符串的样例 '''
from saf import WebDriver, By, DdddOcr
from saf.utils.elementUtils import find_element

def image2str(driver: WebDriver = None, by: str = By.XPATH,value: str = ''):
    '''
    验证码识别工具
    :param driver   : 浏览器驱动
    :param by       : 定位方法
    :param value    : 定位表达式
    :return         : 返回验证码图片中识别的字符串
    '''
    return DdddOcr(show_ad=False).classification(find_element(driver, by, value=value).screenshot_as_base64)


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/20 22:22
@File  : __init__.py
'''

from typing import Union
import datetime

# 将pip更新源设置为：https://pypi.douban.com/simple
# pip config set global.index-url https://pypi.douban.com/simple

from selenium import webdriver as selenium_webdriver
from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys     # 键盘事件
from selenium.webdriver.common.action_chains import ActionChains  # 鼠标事件
from webdriver_manager.chrome import ChromeDriverManager
from loguru import logger
# 运行日志
# logger.add(sink='auto_info_{time}.log', rotation='1 day')

from requests import request, Session
from appium import webdriver as appium_webdriver
from jmespath import search
from yaml import full_load

# 当前版本的httpx，只有在异步请求时才略优于requests，此处可使用到发送消息使用
# from httpx import request
try:
    from ddddocr import DdddOcr
except ImportError as e:
    import os; os.system('pip install ddddocr -i https://pypi.douban.com/simple')
    # 如果运行ddddocr运行报错更新opencv库：
    os.system('pip uninstall opencv-python')
    os.system('pip uninstall opencv-contrib-python')
    os.system('pip install opencv-contrib-python')
    os.system('pip install opencv-python')


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/28 23:23
@File  : sendMsgUtils.py
'''
from saf.data.config import *
from saf import Union, request, search

def sendMessage(
        url: str = None,
        msg: dict = None,
        assert_path: str = None,
        assert_value: Union[str, int] = None
):
    '''
    发送消息函数
    :param url:             WebHook的URL地址
    :param msg:             WebHook发送的信息
    :param assert_path:     判断内容的路径
    :param assert_value:    判断内容的值
    :return:
    '''
    if not url and not msg:
        raise ('url或者msg信息不能为空，必填内容！')
    else:
        response = request(
                        method='POST',
                        url=url,
                        headers={'Content-Type': 'application/json'},
                        json=msg
                    )
    if assert_path and assert_value:
        try:
            assert assert_value == search(assert_path, response.json())
        except AssertionError as e:
            raise ('信息发送失败！')

def feishuWebhookSendMessage(msg:str = '本次测试结束'):
    '''
    使用飞书的webhook发送机器人消息
    :param msg:     需要发送的信息内容，
    :return:

    调用飞书机器人发送飞书群消息
    @feishu_help_document = https://open.feishu.cn/document/ukTMukTMukTM/ucTM5YjL3ETO24yNxkjN
    '''
    sendMessage(url=feishu.webhook(),
                 msg={
                     'msg_type': 'text',
                     'content': {
                         'text': msg
                     }
                 },
                 assert_path='StatusCode',
                 assert_value=0
    )

def dingdingWebhookSendMessage(msg:str = '本次测试结束'):
    '''
    使用钉钉的webhook发送机器人消息
    :param msg:     需要发送的信息内容，
    :return:

    调用钉钉机器人发送钉钉群消息
    @dingding_help_document = https://open.dingtalk.com/document/group/custom-robot-access
    '''
    sendMessage(url=dingding.webhook(),
                 msg={'msgtype': 'text',
                        'text': {
                         'content': msg
                        }
                 },
                 assert_path='errcode',
                 assert_value=0
    )

def robotSendMessage(robot_name: str= 'feishu', msg: str = '本次测试结束'):
    ''' 统一调用机器人发送消息 '''
    if 'feishu' in robot_name:
        feishuWebhookSendMessage(msg)
    if 'dingding' in robot_name:
        dingdingWebhookSendMessage(msg)


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/24 3:24
@File  : setup.py
'''

from setuptools import setup, find_packages
from saf.__version__ import __version__

f = open('README.md', 'r', encoding='utf-8')

setup(
    # 指定项目名称，我们在后期打包时，这就是打包的包名称，当然打包时的名称可能还会包含下面的版本号哟~
    name='xiaobaisaf',
    # 指定版本号
    version=__version__,
    # 这是对当前项目的一个描述
    description="simple_automation_framework(简称：SAF)" +\
                "使用最简单的模式就可以实现需要功能和测试效果，也是xiaobaiauto2的简化版" +\
                "SAF继承了selenium、requests/httpx、appium、loguru、xiaobaiauto2、飞书机器人、钉钉机器人、企业微信机器人（暂时不支持）、禅道提单API",
    long_description=f.read(),
    long_description_content_type="text/markdown",
    # 作者是谁，
    author='xiaobaiTser',
    # 作者的邮箱
    author_email='807447312@qq.com',
    # 写上项目的地址。
    url='https://gitee.com/xiaobaikeji/simlpe_automation_framework',
    # 指定包名，即你需要打包的包名称，要实际在你本地存在哟，它会将指定包名下的所有"*.py"文件进行打包哟，但不会递归去拷贝所有的子包内容。
    # 综上所述，我们如果想要把一个包的所有"*.py"文件进行打包，应该在packages列表写下所有包的层级关系哟~这样就开源将指定包路径的所有".py"文件进行打包!
    keywords="saf test auto automation xiaobai xiaobaiauto2 test framework",
    packages=find_packages(),
    # 指定Python的版本
    python_requires='>=3.5, <3.10',
    install_requires=[
        'allure-pytest',
        'ddddocr',
        'loguru',
        'httpx',
        'jira',
        'webdriver_manager',
        'xiaobaiauto2',
    ],
    entry_points={
        'console_scripts': [
            'xiaobaicmd = saf.utils.xiaobaicmd:main'
        ]
    },
)

'''
#python setup.py sdist bdist_wheel
#python -m twine upload dist/*
'''


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/28 23:25
@File  : zenTao.py
'''
from saf.data.config import *
from saf import Session, Union
import datetime

session = Session()

def loginZenTao():
    ''' 禅道的免密登录，使用session模式登录 '''
    global session
    # 创建应用免密模式
    url = f'{zenTao.baseURL()}/api.php?m=user&f=apilogin&account={zenTao.account()}&code={zenTao.getCode()}&time={zenTao.getTime()}&token={zenTao.getToken()}'
    session.get(url)


def addZenTaoBUG(product: int = 1,
                      branch: int = '',
                      module: int = '',
                      title: str = f'BUG标题-{int(time.time())}',
                      openedBuild: Union[int, str] = 'trunk',
                      execution: int = 0,
                      assignedTo: str = '',
                      deadline: datetime.date = datetime.date.today() + datetime.timedelta(3),  # 为期3天
                      feedbackBy: str = '',
                      type: str = '',
                      os_name: str = '',
                      browser: str = '',
                      color: str = '',
                      serverity: int = 3,
                      pri: int = 3,
                      steps: str = '',
                      story: int = '',
                      task: int = '',
                      mailto: str = '',
                      keywords: str = ''
                      ):
    '''
    禅道提BUG单
    :param product          : 所属产品ID *必填
    :param branch           : 分支/平台
    :param module           : 所属模块
    :param title            : Bug标题 *必填
    :param openedBuild      : 影响版本 *必填
    :param execution        : 所属执行 为0
    :param assignedTo       : 指派给
    :param deadline         : 截止日期 日期格式：YY-mm-dd，如：2022-08-28
    :param type             : Bug类型 取值范围： | codeerror | config | install | security | performance | standard | automation | designdefect | others
    :param os_name          : 操作系统 取值范围： | all | windows | win10 | win8 | win7 | vista | winxp | win2012 | win2008 | win2003 | win2000 | android | ios | wp8 | wp7 | symbian | linux | freebsd | osx | unix | others
    :param browser          : 浏览器 取值范围： | all | ie | ie11 | ie10 | ie9 | ie8 | ie7 | ie6 | chrome | firefox | firefox4 | firefox3 | firefox2 | opera | oprea11 | oprea10 | opera9 | safari | maxthon | uc | other
    :param color            : 标题颜色 颜色格式：#RGB，如：#3da7f5
    :param serverity        : 严重程度 取值范围：1 | 2 | 3 | 4
    :param pri              : 优先级 取值范围：0 | 1 | 2 | 3 | 4
    :param steps            : 重现步骤
    :param story            : 需求ID
    :param task             : 任务ID
    :param mailto           : 抄送给 填写帐号，多个账号用','分隔。
    :param keywords         : 关键词
    :return:

    帮助文档：https://www.zentao.net/book/zentaopmshelp/integration-287.html
    '''
    loginZenTao()
    add_bug_url = f'{zenTao.baseURL()}/bug-create-{product}-{branch}-moduleID={module}.json?tid=h96emyim'
    payload = f"------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"product\"\r\n\r\n{product}\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"module\"\r\n\r\n{module}\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"project\"\r\n\r\n1\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"execution\"\r\n\r\n{execution}\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"openedBuild[]\"\r\n\r\n{openedBuild}\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"assignedTo\"\r\n\r\n{assignedTo}\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"deadline\"\r\n\r\n{str(deadline)}\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"feedbackBy\"\r\n\r\n{feedbackBy}\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"notifyEmail\"\r\n\r\n\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"type\"\r\n\r\n{type}\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"os\"\r\n\r\n{os_name}\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"browser\"\r\n\r\n{browser}\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"title\"\r\n\r\n{title}\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"color\"\r\n\r\n{color}\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"severity\"\r\n\r\n{serverity}\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"pri\"\r\n\r\n{pri}\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"steps\"\r\n\r\n{steps}\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"story\"\r\n\r\n{story}\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"task\"\r\n\r\n{task}\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"oldTaskID\"\r\n\r\n0\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"mailto[]\"\r\n\r\n{mailto}\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"keywords\"\r\n\r\n{keywords}\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"status\"\r\n\r\nactive\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"issueKey\"\r\n\r\n\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"labels[]\"\r\n\r\n\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"files[]\"\r\n\r\n\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"uid\"\r\n\r\n\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"case\"\r\n\r\n0\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"caseVersion\"\r\n\r\n0\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"result\"\r\n\r\n0\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs\r\nContent-Disposition: form-data; name=\"testtask\"\r\n\r\n0\r\n------WebKitFormBoundaryRWJBJ0CsyWBBWFKs--".encode(
        'UTF-8')
    headers = {
        'Accept': 'application/json, text/javascript, */*; q=0.01',
        'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6',
        'Connection': 'keep-alive',
        'Content-Type': 'multipart/form-data; boundary=----WebKitFormBoundaryRWJBJ0CsyWBBWFKs',
        'Referer': f'{zenTao.baseURL()}/bug-create-{product}-{branch}-moduleID={module}.json?tid=h96emyim',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/104.0.5112.102 Safari/537.36 Edg/104.0.1293.63',
        'X-Requested-With': 'XMLHttpRequest'
    }
    session.post(url=add_bug_url, headers=headers, data=payload)


def view_bug(id: int = 1):
    loginZenTao()
    view_bug_url = f'{zenTao.baseURL()}/bug-view-{id}.json?tid=gz3p07qa'
    session.get(url=view_bug_url)